/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.core.dom;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import benten.core.CorePlugin;

/**
 * DOM 要素のデリゲート抽象クラスです。
 */
abstract public class ElementDelegate {

	/**
	 * オリジナルの DOM 要素を取得します。
	 * @return DOM 要素
	 */
	public Element getElement() {
		return element;
	}

	/**
	 * 要素が持つテキスト・ノードの値を取得します。
	 * @return テキスト・ノードの値
	 */
	public String getValue() {
		Node child = element.getFirstChild();
		if (child == null) {
			return "";
		}
		return child.getNodeValue();
	}

	/**
	 * テキスト・ノードの値を設定します。
	 * @param value テキスト・ノードの値
	 */
	public void setValue(String value) {
		Node child = element.getFirstChild();
		if (child == null) {
			throw new IllegalStateException("Element#getFirstChild is null.");
		}
		child.setNodeValue(value);
	}

	/**
	 * DOM 要素デリゲートのリストを DOM ノード・リストに変換します。
	 * @param list DOM 要素デリゲートのリスト
	 * @return DOM ノード・リスト
	 */
	public static List<Element> toElementList(List<? extends ElementDelegate> list) {
		List<Element> elementList = new LinkedList<Element>();
		for (ElementDelegate facade : list) {
			elementList.add(facade.getElement());
		}
		return elementList;
	}

	//-------------------------------------------------------------------------------
	// protected

	protected final Element element;

	/**
	 * コンストラクター。
	 * @param element オリジナル DOM 要素
	 */
	protected ElementDelegate(Element element) {
		this.element = element;
	}

	/**
	 * 属性値または子要素のテキスト・ノードの値を取得します。
	 * このメソッドは getValue(xpath, "") と同じです。
	 * @param xpath XPATH
	 * @return 属性値または子要素のテキスト・ノードの値
	 */
	protected String getValue(String xpath) {
		return getValue(xpath, "");
	}

	/**
	 * 属性値または子要素のテキスト・ノードの値を取得します。
	 * @param xpath XPATH
	 * @param defaultValue 値を取得するできなかった場合のデフォルト値
	 * @return 属性値または子要素のテキスト・ノードの値
	 */
	protected String getValue(String xpath, String defaultValue) {
		Element ele = element;
		for (String name : xpath.split("/")) {
			if (name.startsWith("@")) {
				String value = ele.getAttribute(name.substring(1));
				return value == null ? defaultValue : value;
			}
			Element e = getChildElement(ele, name);
			if (e == null) {
				return defaultValue;
			}
			ele = e;
		}
		Node child = ele.getFirstChild();
		if (child == null) {
			return defaultValue;
		}
		String value = child.getNodeValue();
		return value == null ? defaultValue : value;
	}

	/**
	 * 属性値または子要素のテキスト・ノードの値を設定します。
	 * @param xpath XPATH
	 * @param value テキスト・ノードの値
	 */
	protected void setValue(String xpath, String value) {
		LinkedList<String> pathList = new LinkedList<String>(Arrays.asList(xpath.split("/")));
		String attributeName = null;
		if (pathList.getLast().startsWith("@")) {
			attributeName = pathList.removeLast().substring(1);
		}
		Element ele = element;
		for (String name : pathList) {
			ele = getChildElement(ele, name);
			if (ele == null) {
				throw new IllegalArgumentException("不正な xpath: " + xpath);
			}
		}
		if (attributeName != null) {
			ele.setAttribute(attributeName, value);
		} else {
			ele.getFirstChild().setNodeValue(value);
		}
	}
	/**
	 * タグ名を指定して、この要素直下の DOM 要素デリゲートのリストを取得します。
	 * @param clazz 要素クラス
	 * @param <T> 型パラメーター
	 * @param tagName タグ名
	 * @return DOM 要素デリゲートのリスト
	 */
	protected <T extends ElementDelegate> List<T> getList(Class<T> clazz, String tagName) {
		NodeList nodeList = element.getChildNodes();
		List<T> list = new LinkedList<T>();

		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				Element element = (Element) node;
				if (element.getTagName().equals(tagName)) {
					try {
						T facade = clazz.getConstructor(Element.class).newInstance(element);
						list.add(facade);
					} catch (Exception e) {
						CorePlugin.log(e);
					}
				}
			}
		}
		return list;
	}

	/**
	 * 指定したタグを持つノードを取得します。
	 * @param tagName タグ名
	 * @return ノード。存在しない場合は null。
	 */
	protected Node getChildElement(String tagName) {
		return getChildElement(element, tagName);
	}

	//-------------------------------------------------------------------------------
	// private

	/**
	 * 指定したタグを持つノードを取得します。
	 * @param ele エレメント
	 * @param tagName タグ名
	 * @return ノード。存在しない場合は null。
	 */
	private Element getChildElement(Element ele, String tagName) {
		Element e = null;
		NodeList nodeList = ele.getChildNodes();
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				Element element = (Element) node;
				if (element.getTagName().equals(tagName)) {
					e = (Element) node;
					break;
				}
			}
		}
		return e;
	}
}
